/**************************************************************
	LCD Serial communication monitor (inexpensive)
	SC1602BS 16 columns 2 lines 
	 for PIC16F648A/628(A)
 **************************************************************
	author air_variable
	http://www.ne.jp/asahi/air/variable/
	version 1.00
	Tools:
	MPLAB v8.70
	Hitech PIC C pro9.81
---------------------------------------------------------------
	This program is old header for Hitech PICC.
	if you need re-complie, please set the macro of _LEGACY_HEADERS or
	use the legacy compiler of less or equal version 9.80.
---------------------------------------------------------------
Date	:Time	Version	Build	Note
---------------------------------------------------------------
Date	:Time	Version	Build	Note
20071209:0700	2.00	---		begin the program
20071209:1330	2.01	---		Change EEPROM address
20080112:0010	2.02	---		Fix EEPROM address
20081106:0730	3.00	---		Correspond Hitech PIC C pro9.60pl4 build 2729 or higher
20081106:2100	3.10	---		Include data buffer (125 characters for Hex mode)
20081201:2100	3.11	---		modification lcd.c
20081207:2220	3.20	---		modification lcd.c and main.c
20091117:0317	3.21	---		Fix lcd.c
20110620:0100	3.30	---		corresponding another control code
---------------------------------------------------------------
	Pin assignment
	1:RA2:O:LCD DB6
	2:RA3:O:LCD DB7
	3:RA4:I:GND
	4:MCLR:I:PICKIT2-1(W/10kPU)
	5:VSS:GND
	6:RB0:I:MODE SWITCH
	7:RB1:I:Recive serial
	8:RB2:O:Transmit serial(Not used)
	9:RB3:O:PWM out(Use negative potential)

	18:RA1:O:LCD DB5
	17:RA0:O:LCD DB4
	16:CLOCK:8MHz Resonator1
	15:CLOCK:8MHz Resonator2
	14:VDD:Power 3 to 5 Volt
	13:RB7:O:LCD E
	12:RB6:O:LCD RS
	11:RB5:I:RIGHT SW
	10:RB4:I:LEFT SW
---------------------------------------------------------------
	Display cursor 
	ASCII mode : cursor is blinking (always can switch with command Ctrl-Q/W)
	Binary mode: cursor is stacking (under ber)

	Behavior when code is received
	ascii code	12 		(ctrl +l)	LCD clear
	ascii code	0x0b	(ctrl +k)	Toggle line
	ascii code	0x0d	(Enter)		Return command
	ascii code	0x11	(ctrl +q)	Cursor off
	ascii code	0x17	(ctrl +w)	Cursor on
	ascii code	0x20 to 0xff		Display the corresponding ascii code
---------------------------------------------------------------
Be careful!
	This use LCD character module(SC242A(24 x 2 lines)) is different to power connection of SC1602 series.
	Please careful that the direction of module power polar. 
	
**************************************************************/
#include <htc.h>
#include "sci.h"
#include "lcd.h"

#define MHz 000000
#define _XTAL_FREQ 8MHz


#define	VersionInfo	"Ver 3.30"
#define	SW_MODE		RB0 //mode set
#define	SW_LEFT		RB4 //left key
#define	SW_RIGHT	RB5 //right key
#define DisplayMode	0x7f //eeprom display address*2.02
#define COM_Speed	0x7e //eeprom display address*2.02

//speed
#define Speed1200	250
#define Speed2400	251
#define Speed4800	252
#define Speed9600	253
#define Speed19200	254
#define Speed38400	255

#define MIN_Speed	250
#define MAX_Speed	255


#define ASCIImode	0xff
#define HEXmode		0xfe

//LCD location parameter
//LCD drive for SC1602BS 16 columns 2 lines 
#define	LCD_LineNumber1	0x00
#define	LCD_LineNumber2	0x40
#define	LCD_CursorBlink	0x0f
#define	LCD_CursorOn	0x0e
#define	LCD_CursorOff	0x0c


#define MAX_BUFF	64	//Ring buffer size 2^n

//__CONFIG(0x3F02);

__CONFIG(

	UNPROTECT & LVPDIS & BORDIS & MCLRDIS & PWRTEN & WDTDIS & HS

	);
//Global settings
//Fifo Data
unsigned char Buffer_FIFO[MAX_BUFF];
unsigned char FIFO_write_pointer=0;
unsigned char FIFO_read_pointer=0;
unsigned char FIFO_datasize=0;

void interrupt handler( void )	//interrupt handle
{
	unsigned char temp;
	unsigned char data;
	//Handle RS232 Recieve interrupt
	if( RCIF == 1 ){
		//recive to data
		data = sci_GetByte();
		//set to FIFO
		RCIF = 0; //clear RS232 Recieve interrupt bit
		if(FIFO_datasize<MAX_BUFF){
			Buffer_FIFO[FIFO_write_pointer]=data;
			FIFO_write_pointer=(FIFO_write_pointer+1) & (MAX_BUFF-1);//ring buff 
			FIFO_datasize++;
		}
	}
	if( T0IF==1 ){	//timer interval interrupt
		if (!SW_LEFT){
			while(!SW_LEFT);// button release check
			asm("goto 0");//reset
		}
	
		if (!SW_RIGHT)	{
			temp = EEPROM_READ(DisplayMode);
			if (temp != ASCIImode ) EEPROM_WRITE(DisplayMode,ASCIImode);
				else EEPROM_WRITE(DisplayMode,HEXmode);
			while(!SW_RIGHT);// button release check
			asm("goto 0");//reset
		}

		if (sci_CheckOERR()) { //sci error
			CREN=1;
		}

	//timer interval
	T0IF =0 ;
	}
}

unsigned char Buffer_readdata(void){
	unsigned char wdat;
	//read FIFO buffer
	while(FIFO_datasize==0);	// pause if buffer is empty
	wdat=Buffer_FIFO[FIFO_read_pointer];
	FIFO_read_pointer=(FIFO_read_pointer+1)&(MAX_BUFF-1);//ring buff
	FIFO_datasize--;
	return wdat;//get wdat
//	return FIFO_datasize;//debug
}	

void Buffer_check(void){
	unsigned char data;
	if (FIFO_datasize==MAX_BUFF-1) {
		lcd_putch(0xFF);
		lcd_puts("Full");
		lcd_putch(0xFF);
		RCIE = 0;	//terminal interrupt disable
		FIFO_datasize 		=0;	//FIFO clear
		FIFO_write_pointer 	=0;
		FIFO_read_pointer 	=0;
		CREN =0;	//Receive was aborted
		while (RCIF ==1){
			data = sci_GetByte();		//Dummy read
		}
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		__delay_ms(50);
		CREN = 1;	//Receive is start
		RCIE = 1;	//terminal interrupt enable				
	}	
}

void delay_10ms(unsigned int temp){	//10ms multiple x times
	while(temp--){
		__delay_ms(10);
	}	
}
	

//------------------main
main()
{
// Register
	unsigned int speed = 38400; //default communication speed set
	unsigned char temp,data,buffspeed;
//	Port initalize
//		  	  76543210
	PORTA = 0b11100000;	//Port Initalize
	PORTB = 0b00000000;	//Port Initalize
	TRISA = 0b11110000; //
	TRISB = 0b00110111; //
	CMCON = 0x07;		//PORTA is all digital I/O

//Init
	OPTION = 0b00000111; // prescale 1/256 RBPU= EN
	GIE  = 0;//Int off

//timer0 setting (interrupt event timer 3.328ms)
	T0IE = 0;//timer interrupt is off
	TMR0 = 0;//interrupt timer counter set

//PWM initialize setting (83.8KHz on RB3 @8MHz)
	CCP1CON = 0b00001100;// use PWM mode
	PR2 = 0x17; // resolution is 6.5bit mode
	CCPR1L = 0x0c; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1
/*
//PWM test (31.25KHz on RB3 @8MHz) test only
	CCP1CON = 0b00001100;// use PWM mode
	PR2 = 0xff; // resolution is 8bit mode
	CCPR1L = 0x20; // duty is 50%
	T2CON = 0b00000100;	// POSTSCALE 1:1 ,TMR2 ON , PRESCALE 1:1
*/

//LCD init
//	DelayMs(200);	//stabilize time
	delay_10ms(20);
	lcd_init();		//lcd initalize
	lcd_clear();	//lcd clear
//setting mode
	if (!SW_MODE ){
		lcd_write(LCD_CursorOff);
		lcd_goto(LCD_LineNumber1);
		lcd_puts(VersionInfo);	//version visible
		lcd_goto(LCD_LineNumber2);
		lcd_puts("Set  DOWN  UP");
		while(!SW_MODE);
		buffspeed = EEPROM_READ(COM_Speed);
		while(1){
			__delay_ms(50);		//DelayMs(50);
			lcd_goto(LCD_LineNumber1);
			switch (buffspeed){
				case Speed1200:
					lcd_puts(" 12");
					break;
				case Speed2400:
					lcd_puts(" 24");
					break;
				case Speed4800:
					lcd_puts(" 48");
					break;
				case Speed9600:
					lcd_puts(" 96");
					break;
				case Speed19200:
					lcd_puts("192");
					break;
				case Speed38400:
					lcd_puts("384");
					break;
				default: // data is broken
					buffspeed =Speed38400;
			}
			lcd_puts("00bps");
			
			//key check
			if (!SW_LEFT ) {
				if ( buffspeed > MIN_Speed) buffspeed--;
				while(!SW_LEFT);

			}
			if (!SW_RIGHT  ){
				if ( buffspeed < MAX_Speed) buffspeed++;
				while(!SW_RIGHT);

			}
			if (!SW_MODE  ){
				if (buffspeed != EEPROM_READ(COM_Speed)){
					EEPROM_WRITE(COM_Speed,buffspeed);
					lcd_goto(LCD_LineNumber2);
					lcd_puts("OK ");
					lcd_write(LCD_CursorBlink);
					//DelayMs(255);
					delay_10ms(50);	
				}
				while(!SW_MODE);
				break;
			}
		}
	}

	buffspeed = EEPROM_READ(COM_Speed);
//sci initialize
	switch (buffspeed){
		case Speed1200:
			speed = 1200;
			break;
		case Speed2400:
			speed = 2400;
			break;
		case Speed4800:
			speed = 4800;
			break;
		case Speed9600:
			speed = 9600;
			break;
		case Speed19200:
			speed = 19200;
			break;
		case Speed38400: //This speed is not accept HEX mode.
			speed = 38400;
			break;
		default:
			speed = 38400;
	}

	sci_Init(speed ,SCI_EIGHT);// Baud set and Bit set

//interrupt enable
//	INTEDG = 0; //RB0 Key interrupt fall edge
//	INTE = 1;//RB0 is interrupt mode
	T0IE = 1;//timer interrupt on
	RCIE = 1;//recive interrupt on
	PEIE = 1;//Peripheral interrupt Enable 
	GIE  = 1;//Global interrupt Enable 

//sci check
	if (sci_CheckOERR()) { //sci error
		while (!TRMT);
		CREN=1;
	}

//Main routiene
	lcd_init();
	lcd_clear();	//lcd clear
	temp = EEPROM_READ(DisplayMode);

	switch (temp){
		case HEXmode :
			lcd_write(LCD_CursorOn);	//Init cursor
			//ASCII to HEX convert(Use less or equal 19200bps)
//			lcd_puts("HEX:");
//			lcd_goto(LCD_LineNumber2);
			while(1)	{
				data= Buffer_readdata();//Buffer read 
			    //Send high nibble
			    if( (data>>4) > 9 )
			        lcd_putch( 'A' - 10 + (data>>4) );
			    	else lcd_putch( '0' + (data>>4) );
			    //Send low nibble
			    if( (data&0x0F) > 9 )
			        lcd_putch( 'A' - 10 + (data&0x0F) );
			    	else lcd_putch( '0' + (data&0x0F) );

				Buffer_check();	//Buffer check
			}
			break;	

		default:
			lcd_write(LCD_CursorBlink);	//Init cursor
//			lcd_puts("ASCII:");
//			lcd_goto(LCD_LineNumber2);
			while(1){
				data= Buffer_readdata();//Buffer read 
				switch ( data ){
					case 12	:	//clear
						lcd_clear();
						break;
					case 0x0b	:	//Verticcal tab
						lcd_newline(1);
						break;
					case 0x0d	:	//return command
						lcd_newline(0);
						break;
					case 0x11	:	//cursor off(ctrl +q)
						lcd_write(LCD_CursorOff);//cursor off
						break;
					case 0x17	:	//cursor on(ctrl +w)
						lcd_write(LCD_CursorBlink);//cursor on
						break;
					default		:
						if 	(( data >= 0x20 ) && ( data <= 0xff  )) lcd_putch(data);
						break;
				}
			}
		break;

	}

}
